<?php

namespace FiloBlu\OrderStatus\Observer;

use \Magento\Framework\Event\ObserverInterface;
use \Magento\Framework\Event\Observer;
use \Magento\Framework\App\Config\ScopeConfigInterface;
use \Magento\Store\Model\ScopeInterface;

/**
 * Observer on Adyen notification and set order status accordingly
 * This observer needs the adyen module to be patched with adyen_notification_events.patch
 */
 
 class UpdateStatus implements ObserverInterface {
    
    /**
     * @var \Psr\Log\LoggerInterface $logger
     */  
    protected $_logger;
    
    protected $_config;
    
    const MANAGED_NOTIFICATION_CODES = array('AUTHORISATION', 'CAPTURE', 'MANUAL_REVIEW_ACCEPT');
    const WHITELISTED_METHODS = array('adyen_cc', 'adyen_oneclick', 'adyen_google_pay', 'adyen_apple_pay');
    const WHITELISTED_METHOD_TYPES = array('bankTransfer_IBAN');
    const AUTHORIZED_ORDER_STATUS = 'confirmed';
    const AUTHORIZED_ORDER_STATUS_CONFIG_PATH = 'payment/adyen_abstract/payment_pre_authorized';
    const CAPTURED_ORDER_STATUS = 'payment_confirmed';
    const CAPTURED_ORDER_STATUS_CONFIG_PATH = 'payment/adyen_abstract/payment_authorized';
    const DEBUG_LOG_ACTIVE = TRUE;
	
    /**
     * @param \Psr\Log\LoggerInterface $logger
     */  
	public function __construct(
		\Psr\Log\LoggerInterface $logger,
                ScopeConfigInterface $config
	){
		$this->_logger = $logger;
                $this->_config = $config;
	}

    /**
     * Process order
     *
     * @param \Magento\Framework\Event\Observer $observer
     */
    public function execute(Observer $observer) {
        /* @var $order \Magento\Sales\Model\Order */
        if (self::DEBUG_LOG_ACTIVE) {
            $this->_logger->info("FiloBlu\OrderStatus\Observer: Processing Notification");
        }
        $notification_code = $observer->getEvent()->getNotificationCode();
        if (!in_array($notification_code, self::MANAGED_NOTIFICATION_CODES)) {
            if (self::DEBUG_LOG_ACTIVE) {
                $this->_logger->info("FiloBlu\OrderStatus\Observer: Skipped notification");
            }
            return;
        }
        if ($notification_code == 'AUTHORISATION' && $observer->getEvent()->getManualReview()) {
            if (self::DEBUG_LOG_ACTIVE) {
                $this->_logger->info("FiloBlu\OrderStatus\Observer: Skipped notification due to Manual review flag");
            }
            return;
        }
        $order = $observer->getEvent()->getOrder();
        $payment_method = $order->getPayment()->getMethod();
        $payment_method_type = $order->getPayment()->getPaymentMethodType();
        if (self::DEBUG_LOG_ACTIVE) {
            $this->_logger->info("FiloBlu\OrderStatus\Observer: Processing Order");
            $this->_logger->info("FiloBlu\OrderStatus\Observer: payment_method: $payment_method");
            $this->_logger->info("FiloBlu\OrderStatus\Observer: payment_method_type: $payment_method_type");
        }
        if (in_array($payment_method, self::WHITELISTED_METHODS)
                || in_array($payment_method_type, self::WHITELISTED_METHOD_TYPES)) {
            if ($order->hasInvoices() && $notification_code == 'CAPTURE') {
                $captured_order_status = $this->_config->getValue(self::CAPTURED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) ? 
                        $this->_config->getValue(self::CAPTURED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) : self::CAPTURED_ORDER_STATUS;
                if (self::DEBUG_LOG_ACTIVE) {
                    $this->_logger->info("FiloBlu\OrderStatus\Observer: Order has invoices and notification is CAPTURE.");
                }
                $order->addStatusHistoryComment(
                    __('Order processed by mod2-order-status'), $captured_order_status);
                if (self::DEBUG_LOG_ACTIVE) {
                    $this->_logger->info("FiloBlu\OrderStatus\Observer: Order status set to " . $captured_order_status);
                }
            } elseif (($notification_code == 'AUTHORISATION' || $notification_code == 'MANUAL_REVIEW_ACCEPT') && $order->getCustomerEmail() != 'sistemi@filoblu.com') {
                $authorized_order_status = $this->_config->getValue(self::AUTHORIZED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) ? 
                        $this->_config->getValue(self::AUTHORIZED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) : self::AUTHORIZED_ORDER_STATUS;
                if (self::DEBUG_LOG_ACTIVE) {
                    $this->_logger->info("FiloBlu\OrderStatus\Observer: Order has no invoices.");
                }
                $order->setState(\Magento\Sales\Model\Order::STATE_PROCESSING)
                        ->addStatusHistoryComment(
                    __('Order processed by mod2-order-status'), $authorized_order_status);
                if (self::DEBUG_LOG_ACTIVE) {
                    $this->_logger->info("FiloBlu\OrderStatus\Observer: Order status set to " . $authorized_order_status);
                }
            } else {
                if (self::DEBUG_LOG_ACTIVE) {
                    $this->_logger->info("FiloBlu\OrderStatus\Observer: Skipping order modification due to conditions not met.");
                }
            }
        }
        return TRUE;
    }
}